#!/bin/sh
#\
exec stash $0 $@

# last edition 05-01-2005

load_package gap
source /my/modules/directory/GenALA/bin/gap4.inc

# initialize C and Tcl tag ID and tag type arrays
# InitTagArray         ;# if work includes usage of tag information
# InitCanvasConstants  ;# canvas!
# InitLists            ;# initialize built-in lists
# InitListTrace        ;# display of lists

# turn off the annoying bell
error_bell 0

# how can we get the script path???
# set MyDir $0
# puts $MyDir

# parameters from command line arguments
if {$argc != 2} {
  puts stderr "USAGE"
  puts stderr " assembledirected.tcl  GapDB.V fofn"
  puts stderr ""
  puts stderr "ARGUMENTS"
  puts stderr " fofn       file of filenames which are meant to be assembled. The"
  puts stderr "            file of filenames and the Experiment files must reside"
  puts stderr "            in the same directory."
  exit 1
}
set DbSpec [lindex $argv 0]
set fofn   [lindex $argv 1]
if {! [file exists $fofn]} {
  puts stderr "ERROR: unable to open file of filenames $fofn"
  exit 1
}


################################################################################
# work
################################################################################

set io [OpenDatabaseRWc $DbSpec]


# list DB parameters
set DbInfo [io_read_database $io]
set NumContig [keylget DbInfo num_contigs]
set NumRead   [keylget DbInfo num_readings]
puts "number of contigs/reads: $NumContig/$NumRead"

# check for initial errors
if {$NumRead == 0} {
  set ErrorBefore "Database is logically consistent\n"
} else {
  capture {check_database -io $io} ErrorBefore
}
set ErrorBeforeNum 0
while {[regsub "\n" "$ErrorBefore" a ErrorBefore]} {
  incr ErrorBeforeNum
}
set ErrorBeforeNum [expr $ErrorBeforeNum - 1]

# change to read directory
# read file of read names
set DirSave [pwd]
if {[set CtPos [string last / $fofn]] == -1} {
  set DirFofn "./"
} else {
  set DirFofn [string range $fofn 0 [expr $CtPos-1]]
  set fofn    [string range $fofn [expr $CtPos+1] end]
}
catch {cd $DirFofn} err
if {$err != ""} {
  puts stderr "ERROR: unable to change to reading directory $DirFofn"
  close_db -io $io
  exit 1
}
set ListToAdd [PlainToList [exec cat $fofn]]

# we need to adjust maxdb (number of reads)
# although the default value of 8000 would work for most (but not all) cases
# we need the standard value of #reads + #contigs plus a small joker
set maxdb  [expr round (($NumRead + 100 + [llength $ListToAdd]) * 1.2)]
# We also need to adjust maxseq. Function assemble_direct should do it
#   for us (GAP4.4), but this doesn't seem to work in all cases. We adjust
#   to maximum number of nucleotides expected to be reached throughout after
#   assembly plus a joker.
# possibly the joker has to be risen.
set maxseq [expr round ((([db_info t_contig_length $io] + 1000) + 1000 * [llength $ListToAdd]) * 1.3)]
#puts "maxdb/maxseq: $maxdb/$maxseq"

# prepare for import
# define import batch size
set ReloadSize 4000
set CtReload 0
set ListFail ""
for {set CtCycleHave 0} {$CtCycleHave < [llength $ListToAdd]} {set CtCycleHave [expr $CtCycleHave+$ReloadSize]} {
  set ListCycle [lrange $ListToAdd $CtCycleHave [expr $CtCycleHave+$ReloadSize-1]]
  if {$ListCycle == ""} { break }
  puts "\nassembling batch of reads [expr $CtCycleHave+1] to [expr $CtCycleHave + [llength $ListCycle]]"

  # import readings
  set mismatch  -1;   # don't check for mismatch
  set out_mode   0;   # no output
  set ListFail [concat $ListFail [assemble_direct  \
    -io $io                   \
    -files $ListCycle         \
    -max_pmismatch $mismatch  \
    -output_mode $out_mode    \
    ]]

  # eventually: close database, reopen
  # database parameters maxdb and maxseq will still be active
  if {[expr $CtCycleHave + [llength $ListCycle] - $CtReload * $ReloadSize] >= $ReloadSize} {
    incr CtReload
    puts "closing/reopening database"
    close_db -io $io
    cd $DirSave
    set io [OpenDatabaseRWc $DbSpec]
    cd $DirFofn
  }
}

# check result of procedure
if {$ListFail != ""} {
  puts "reads that didn't enter the database:"
  puts [join $ListFail "\n"]
}
set DbInfo [io_read_database $io]
set NumContig   [keylget DbInfo num_contigs]
set NumReadAdd  [expr [keylget DbInfo num_readings] - $NumRead]
set NumRead     [keylget DbInfo num_readings]
puts "number of contigs/reads: $NumContig/$NumRead"
puts "number of reads added: $NumReadAdd"

# enter contig tags
if {[file exists "ContigTags"]} {
  enter_tags -io $io -file "ContigTags"
}

# check for errors, close the database
capture {puts -nonewline [format "\n%[string length $ErrorBefore]s" ""]} ErrorAfter
capture {check_database -io $io} ErrorAfter
set ErrorAfterNum 0
while {[regsub "\n" "$ErrorAfter" a ErrorAfter]} {
  incr ErrorAfterNum
}
set ErrorAfterNum [expr $ErrorAfterNum - 1]
close_db -io $io
if {$ErrorAfterNum > $ErrorBeforeNum} {
  puts stderr "ERROR: found additional ([expr {$ErrorAfterNum - $ErrorBeforeNum}]) database errors following the procedure"
  exit 1
} elseif {[llength $ListToAdd] && ! [llength $ListFail] && ! $NumReadAdd} {
  puts stderr "ERROR: import procedure failed (error unknown)"
  exit 1
} else {
  exit 0
}
