#!/bin/sh
#\
exec stash $0 $@

# last edition 05-01-2005

load_package gap
source /my/modules/directory/GenALA/bin/gap4.inc

# initialise C and Tcl tag id and tag type arrays
# InitTagArray         ;# if work includes usage of tag information
# InitCanvasConstants  ;# canvas!
# InitLists            ;# initialize built-in lists
# InitListTrace        ;# display of lists

# turn off the annoying bell
error_bell 0

# how can we get the script path???
# set MyDir $0
# puts $MyDir

# parameters from command line arguments
if {$argc != 2} {
  puts stderr "USAGE"
  puts stderr "assemblepreass.tcl  GapDB.V fofn"
  puts stderr ""
  puts stderr "ARGUMENTS"
  puts stderr "fofn        file of filenames which are meant to be assembled. The"
  puts stderr "            files of filenames and the Experiment files must reside"
  puts stderr "            in the same directory."
  exit 1
}
set DbSpec [lindex $argv 0]
set fofn   [lindex $argv 1]
if {! [file exists $fofn]} {
  puts stderr "ERROR: unable to open file of filenames $fofn"
  exit 1
}


################################################################################
# work
################################################################################

# open the database in the context of pwd (changed later)
set io [OpenDatabaseRWc $DbSpec]
# list DB parameters
set DbInfo [io_read_database $io]
set NumContig [keylget DbInfo num_contigs]
set NumRead   [keylget DbInfo num_readings]
puts "number of contigs/reads: $NumContig/$NumRead"

# check for initial errors
if {$NumRead == 0} {
  set ErrorBefore "Database is logically consistent\n"
} else {
  capture {check_database -io $io} ErrorBefore
}
set ErrorBeforeNum 0
while {[regsub "\n" "$ErrorBefore" a ErrorBefore]} {
  incr ErrorBeforeNum
}
set ErrorBeforeNum [expr $ErrorBeforeNum - 1]

# read file of read names in the context of pwd (changed later)
# needed to:
# - read list of Experiment files to be imported
# - estimate of future DB size
set ListToAdd [PlainToList [exec cat $fofn]]
puts "readings to be added: [llength $ListToAdd]"

# scale database
#
set dbscale 1.3
# we need to adjust maxdb (number of reads)
#   Although a default value of 8000 would work for most (but not all) cases
#   we need the standard value of #reads + #contigs plus a small joker,
#   everything scaled by a joker factor.
set maxdb  [expr round (($NumRead + 200 + [llength $ListToAdd]) * $dbscale)]
# we also need to adjust maxdb (number of reads). Function assemble_direct
#   should do it for us (GAP4.4, GAP4.5), but this does not seem to work in all
#   cases. We adjust to an estimate for the maximum number of nucleotides
#   expected to be reached overall, after assembly is done, plus a joker,
#   everything scaled by a joker factor ($dbscale). The joker value is the
#   skrew where to optimise for future needs.
set maxseq [expr round ((([db_info t_contig_length $io] + 1000) + 1000 * [llength $ListToAdd]) * $dbscale)]
puts "maxdb/maxseq: $maxdb/$maxseq"

# change to read directory
set DirSave [pwd]
if {[set CtPos [string last / $fofn]] == -1} {
  set DirFofn "./"
} else {
  set DirFofn [string range $fofn 0 [expr $CtPos-1]]
}
catch {cd $DirFofn} err
if {$err != ""} {
  puts stderr "ERROR: unable to change to reading directory $DirFofn"
  close_db -io $io
  exit 1
}


# cycle for import
# - note: cwd was set to location of fofn = location of reading files
# - define import batch size
set ListBatchSize 4000
set CtReload 0
set ListFail ""
for {set CtCycleHave 0} {$CtCycleHave < [llength $ListToAdd]} {set CtCycleHave [expr $CtCycleHave+$ListBatchSize]} {
  set ListCycle [lrange $ListToAdd $CtCycleHave [expr $CtCycleHave+$ListBatchSize-1]]
  if {$ListCycle == ""} { break }
  puts "\nassembling batch of reads [expr $CtCycleHave+1] to [expr $CtCycleHave + [llength $ListCycle]]"

  # import readings
  set ListFail [concat $ListFail [pre_assemble  \
    -io $io              \
    -files $ListCycle    \
    ]]

  # eventually: close database, reopen
  # database parameters maxdb and maxseq will still be active
  if {[expr $CtCycleHave + [llength $ListCycle] - $CtReload * $ListBatchSize] >= $ListBatchSize} {
    incr CtReload
    puts "closing/reopening database"
    close_db -io $io
    cd $DirSave
    set io [OpenDatabaseRWc $DbSpec]
    cd $DirFofn
  }
}

# check result of procedure
if {$ListFail != ""} {
  puts "reads that didn't enter the database:"
  puts [join $ListFail "\n"]
}
set DbInfo [io_read_database $io]
set NumContig   [keylget DbInfo num_contigs]
set NumReadAdd  [expr [keylget DbInfo num_readings] - $NumRead]
set NumRead     [keylget DbInfo num_readings]
puts "number of contigs/reads: $NumContig/$NumRead"
puts "number of reads added: $NumReadAdd"

# check for errors, close the database
capture {puts -nonewline [format "\n%[string length $ErrorBefore]s" ""]} ErrorAfter
capture {check_database -io $io} ErrorAfter
set ErrorAfterNum 0
while {[regsub "\n" "$ErrorAfter" a ErrorAfter]} {
  incr ErrorAfterNum
}
set ErrorAfterNum [expr $ErrorAfterNum - 1]
close_db -io $io
if {$ErrorAfterNum > $ErrorBeforeNum} {
#  exec mv -f $DbPathName.X     $DbSpec
#  exec mv -f $DbPathName.X.aux $DbSpec.aux
#  exec mv -f $DbPathName.X.log $DbSpec.log
  puts stderr "ERROR: found (additional) database errors after import procedure"
#  puts "GAP project left unchanged"
  exit 1
} elseif {[llength $ListToAdd] && ! [llength $ListFail] && ! $NumReadAdd} {
  puts stderr "ERROR: import procedure failed (error unknown)"
  exit 1
} else {
#  exec rm -f $DbPathName.X
#  exec rm -f $DbPathName.X.aux
#  exec rm -f $DbPathName.X.log
  exit 0
}
