#!/bin/sh
#\
exec stash $0 $@

################################################################################
# intro
################################################################################

load_package gap
source /my/modules/directory/GenALA/bin/gap4.inc

# initialize C and tcl tag id and tag type arrays
#InitTagArray         ;# if work includes usage of tag information
# InitCanvasConstants  ;# canvas!
#InitLists            ;# initialize built-in lists
# InitListTrace        ;# only for display of lists

# turn off the annoying bell
error_bell 0

# consensus calculation parameters
set consensus_mode 2
set consensus_cutoff 0.01
set quality_cutoff -1
  # read quality cutoff remains constant throughout all operations
#set consensus_iub 0

# output directives (may be changed by command line options)
set SeqFmtLib "{staden 1} {fasta 2} {experiment 3} {1 1} {2 2} {3 3}"
set SeqOutFmt 3
set FlagStripPads 0
set SeqMaskSpec ""


# options from command line arguments
while {$argc > 0 && "[string index [lindex $argv 0] 0]" == "-"} {
  # splice option string from array of command line arguments
  set arg [lindex $argv 0]
  set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # stop scanning for options
  if {$arg == "--"} {
    break

  # consensus calculation mode
  } elseif {$arg == "-c"} {
    if {! [regexp {^[0-2]$} [lindex $argv 0]]} {
      puts stderr "ERROR: invalid consensus calculation mode '[lindex $argv 0]'"
      exit 1
    }
    set consensus_mode [lindex $argv 0]
    set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # output format
  } elseif {$arg == "-f"} {
    set SeqFmtSpec [string tolower [lindex $argv 0]]
    set argv [lrange $argv 1 $argc] ; incr argc -1 ;
    if {[keylget SeqFmtLib $SeqFmtSpec SeqOutFmt] == 0} {
      puts stderr "ERROR: unknown sequence format type '$SeqFmtSpec'"
      exit 1
    }

  # usage help
  } elseif {$arg == "-h"} {
    set argc 0
    break

  # mask tags
  } elseif {$arg == "-i"} {
    set consensus_iub 1

  # mask tags
  } elseif {$arg == "-m"} {
    set SeqMaskSpec "[string toupper [lindex $argv 0]]"
    set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # output format
  } elseif {$arg == "-q"} {
    if {! [regexp {^0\.[0-9][0-9]?$} [lindex $argv 0]]} {
      puts stderr "ERROR: invalid consensus quality cutoff value '[lindex $argv 0]'"
      exit 1
    }
    set consensus_cutoff [lindex $argv 0]
    set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # strip pads
  } elseif {$arg == "-s"} {
    set FlagStripPads 1

  # unknown option
  } else {
    puts stderr "ERROR: invalid option '$arg'"
    exit 1
  }
}

# parameters from command line arguments
if {$argc < 2} {
  puts stderr ""
  puts stderr "PURPOSE"
  puts stderr " export consensus sequence(s) from a GAP4 database"
  puts stderr ""
  puts stderr "USAGE"
  puts stderr " exportconsensus.tcl  \[-c N\] \[-f S\] \[-h\] \[-m S\] \[-q F\] \[-s\] \\"
  puts stderr "   GapDB.V ConsFile \[ContigID(s)\]"
  puts stderr ""
  puts stderr "arguments:"
  puts stderr " GapDB.V   path of GAP4 database, including version suffix"
  puts stderr " ConsFile  path of consensus output, existing file will be overwritten."
  puts stderr " ContigID  optional: contig identifiers (numerical or alphanumerical)"
  puts stderr ""
  puts stderr "options:"
  puts stderr " --    stop scanning for options"
  puts stderr " -c N  use consensus calculation mode N:"
  puts stderr "       0  base type frequencies (default)"
  puts stderr "       1  quality weighted base type"
  puts stderr "       2  use confidence values"
  puts stderr " -f S  define output format using one of format specifiers (case-insensitive):"
  puts stderr "       fastA, Staden, Experiment (default)"
  puts stderr " -h    show this command line usage help"
  puts stderr " -i    use IUPAC/IUB codes in consensus if bases in alignment disagree"
  puts stderr " -m S  mask tags. Specify space-delimited list of tag IDs"
  puts stderr "       BUG: CVEC tags won't ever be masked (blame Staden for this!)."
  puts stderr " -q F  set consensus quality cutoff value (default: $quality_cutoff)"
  puts stderr " -s    strip pads (default: don't)"
  puts stderr "       BUG: the fastA format cannot support pads. (blame Staden for this!)."
  puts stderr "         So, pads are always stripped in fastA format output."
  puts stderr ""
  exit 1
}

set DbSpec      [lindex $argv 0]
set FileOut     [lindex $argv 1]
set ListContig  [lrange $argv 2 [expr $argc-1]]


################################################################################
# work
################################################################################

# open the database
set io [OpenDatabaseR $DbSpec]

# We need to adjust maxseq. Otherwise, function get_consensus won't work
#   properly.
set maxseq [expr round ([db_info t_contig_length $io] * 1.2) ]

# non-redundant contig list
if {$ListContig != ""} {
  set ListContig [remove_contig_duplicates -io $io -contigs $ListContig]
} else {
  set ListContig [CreateAllContigList $io]
}

# work around strip-pads bug in fastA format
if {$SeqOutFmt == 2} {
  set FlagStripPads 1
}

# write consensi to file
get_consensus -io $io         \
  -outfile $FileOut           \
  -contigs $ListContig        \
  -format $SeqOutFmt          \
  -strip_pads $FlagStripPads  \
  -annotations 1              \
  -truncate 1                 \
  -mask mask                  \
  -tag_types $SeqMaskSpec     \

# close the database
close_db -io $io
exit
