#!/bin/sh
#\
exec stash $0 $@

# last edition 19-01-2005

load_package gap
source /my/modules/directory/GenALA/bin/gap4.inc

proc PlainToList {ListPlain} {
  set ListData ""

  # remove strings following the read identifier
  foreach line [split $ListPlain "\n"] {
    regsub { .*$} $line {} liner
    lappend ListData $liner
  }

  # return data
  return $ListData
}

InitTagArray         ;# if work includes usage of tag information
InitLists            ;# initialize built-in lists

# turn off the annoying bell
error_bell 0

# consensus calculation parameters
set consensus_mode   2  
set consensus_cutoff 0.01
set quality_cutoff   -1


# options from command line arguments
while {$argc > 0 && "[string index [lindex $argv 0] 0]" == "-"} {
  # splice option string from array of command line arguments
  set arg [lindex $argv 0]
  set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # stop scanning for options
  if {$arg == "--"} {
    break

  # usage help
  } elseif {$arg == "-h"} {
    set argc 0
    break

  # unknown option
  } else {
    puts stderr "ERROR: invalid option '$arg'"
    exit 1
  }
}

# parameters from command line arguments
if {$argc < 2} {
  puts stderr ""
  puts stderr "PURPOSE"
  puts stderr " export directed assembly from a GAP4 database"
  puts stderr ""
  puts stderr "USAGE"
  puts stderr " exportdirected.tcl  GapDB.V extracts \[contig_id(s)\]"
  puts stderr ""
  puts stderr "arguments:"
  puts stderr " GapDB.V    path of GAP4 database, including version suffix"
  puts stderr " extracts   directory path of output, will be created if necessary"
  puts stderr " contig_id  contig selector, alphanumerical or numerical (with leading \"#\")"
  puts stderr ""
  puts stderr "options:"
  puts stderr " --    stop scanning for options"
  puts stderr " -h    show this command line usage help"
  puts stderr ""
  exit 1
}
set DbSpec      [lindex $argv 0]
set ExpFolder   [lindex $argv 1]

if {[set dot [string last . $DbSpec]] == -1} {
  puts stderr "ERROR: invalid database name '$argv'"
  exit 1
}

# DB file should exist
if {! [file exists $DbSpec]} {
  puts stderr "ERROR: specified database '$argv' doesn't exist"
  exit 1
}

  # the folder needn't to be cleaned
  # actually it mustn't be cleaned, cmp. Gap.pl -TransposExp
set ListContig  [lrange $argv 2 [expr $argc-1]]
set dbname [string range $DbSpec 0 [expr $dot-1]]
set dbvers [string range $DbSpec [expr $dot+1] end]


################################################################################
# basic work
################################################################################

# open the database
set io [open_db -name $dbname -version $dbvers -access r]
set DbInfo [io_read_database $io]
set NumContig [keylget DbInfo num_contigs]
set NumRead   [keylget DbInfo num_readings]

# we need to adjust maxdb (number of reads) although the default value
#   of 8000 would work for most (but not all) cases
# we need the standard value of #reads + #contigs plus a small joker
set maxdb  [expr round (($NumRead + 100) * 1.2)]
# We need to adjust maxseq. Otherwise, function get_consensus won't work
#   properly. We adjust to maximum number of nucleotides expected
#   to be reached after complete assembly plus a joker.
set maxseq [expr round (([db_info t_contig_length $io] + 1000) * 1.3) ]

# non-redundant contig list in alphanumeric name syntax
if {$ListContig != ""} {
  set ListContig [remove_contig_duplicates -io $io -contigs $ListContig]
} else {
  set ListContig [CreateAllContigList $io]
}
if { "$ListContig" == "" } {
  close_db -io $io
  set outfoid [open "$ExpFolder/fofn" w]
  close $outfoid 
  exit 0
}

# create list of all readings to export
set ListRead ""
foreach CurrContig $ListContig {
  set ListRead [concat $ListRead [ListGet "\[$CurrContig\]"]]
}
if {! [llength $ListRead]} {
  if {! [file exists $ExpFolder]} {
    exec mkdir "$ExpFolder"
  } elseif {! [file isdirectory $ExpFolder]} {
    puts "ERROR: $ExpFolder already exists and is not a directory"
    exit 1
  }
  # following truncation also works with noclobber shell variable set
  exec echo -n "" > "$ExpFolder/fofn"
  close_db -io $io
  exit 0
}

# export readings
# contig tags are written into the Experiment file for the leftmost reading
#   in the contig
extract_readings      \
  -io $io             \
  -readings $ListRead \
  -format   2


################################################################################
# work on consensus tags
################################################################################
# function 'extract_readings' randomly fails to export the contig tags
# ('TC') correctly. So, we do this work-around here.
# - efficiently remove ALL consensus tags from directed assembly
# - retrieve all contig tags via function get_consensus
#   and store them in a separate file called 'ContigTags'.
#   CVEC tags are missed by this strategy cause they're not exported
#   via function get_consensus (this tag seems to be regarded as a
#   magic tag).
# - retrieve all CVEC contig tags via crawling through the data
#   substructures for each contig. Append tags to the file ContigTags
#   (not ContigConsensi!).
# - the routine for import of a directed assembly will find the contig
#   tags in the special file

# temporary folder
if {[file exists "$ExpFolder.tmp"]} {
  exec rm -fR "$ExpFolder.tmp"
}
exec mkdir "$ExpFolder.tmp"

# loop over Experiment files which contain contig tags
set DirSave [pwd]
set ListRead [PlainToList [exec cat "$ExpFolder/fofn"]]
# there's a problem that Tcl aborts if grep doesn't find anything (exit code x)
# we've encapsulated the grep call into catch { } and now we're testing if the
# process has yielded something useful. If not, $ListHaveTC holds the message
# 'child process exited abnormally'.
foreach CurrRead $ListRead {
  catch { exec grep -l "^TC   " "$ExpFolder/$CurrRead" } HaveTC
  regsub {:} $HaveTC {} HaveTCb
  if {[lindex "$HaveTCb" 0] != "child"} {

    # remove contig tags
    #puts "contig tag removal for $CurrRead ($HaveTCb)"
    exec cp -f "$ExpFolder/$CurrRead" "$ExpFolder.tmp/$CurrRead.A"
    catch { exec grep -v "TC   " "$ExpFolder.tmp/$CurrRead.A" >> "$ExpFolder.tmp/$CurrRead.C" } fake
    exec cp -f "$ExpFolder.tmp/$CurrRead.C" "$ExpFolder/$CurrRead"
  }
}

# function get_consensus writes out contig tags correctly
# don't strip pads!
set out_format 3;  # Experiment file
get_consensus -io $io  \
  -contigs $ListContig  \
  -outfile "$ExpFolder/ContigConsensi"  \
  -strip_pads 0  \
  -annotations 1

# sample contig tags, write to file of contig tags
# It won't work to append the tags to the Experiment files. We would get
# problems for contigs starting with a minus-stranded read
catch { exec /usr/xpg4/bin/grep -E "^(ID|TC)   " "$ExpFolder/ContigConsensi" > "$ExpFolder/ContigTags" } fake

# sample CVEC contig tags
# we could do this procedure for the whole set of consensus tags of all contigs
# - => do a benchmark for this procedure!
# - there's no implementation yet to print the text field of an annotation!
# loop over contigs
set outctags [open "$ExpFolder/ContigTags" a]
set NumEntries [keylget DbInfo num_contigs]
for {set CtContig 1} {$CtContig <= $NumEntries} {incr CtContig} {
  set ContigStruct [io_read_contig $io $CtContig]
  set CtAnnot [keylget ContigStruct annotations]
  
  # contig selected? => print contig ID
  set ReadStruct [io_read_reading $io [keylget ContigStruct left]]
  set ReadID [io_read_text $io [keylget ReadStruct name]]
  if {[lsearch -exact $ListContig $ReadID] == -1} { continue }
  puts $outctags "ID   $ReadID"

  # loop over annotations
  while { $CtAnnot } {
    set AnnotStruct [io_read_annotation $io $CtAnnot]
    if {[keylget AnnotStruct type] == "CVEC"} {
  
      # print CVEC annotation entry
      set PosOff [keylget AnnotStruct position]
      set PosEnd [expr $PosOff + [keylget AnnotStruct length] - 1]
      puts $outctags "TC   CVEC + $PosOff..$PosEnd"
    }
    set CtAnnot [keylget AnnotStruct next]
  }
}
close $outctags


################################################################################
# close the database, tidy up

exec rm -fR "$ExpFolder.tmp"
close_db -io $io
exit 0
