################################################################################
#
#  Tcl Include File for GAP4 Scripting
#
#  all rights reserved
#  Karol Szafranski at IMB Jena, Dept. Genome Analysis, 2000
#  szafrans@imb-jena.de
#
################################################################################
#
#  FUNCTIONS
#
# - database handling
#   OpenDatabaseR    open database in read-only mode
#   OpenDatabaseRW   open existing database for read/write
#   OpenDatabaseRWc  open database for read/write. If it doesn't exist it's
#                    newly created
#
# - report
#   DatabaseInfo     returns a string describing the database
#                    this code is copied from Staden's gap.tcl (version 4.4)
#
# - lists
#   ListGet          get read list from contig list etc., see Staden package
#   PlainToList      convert plain text read list to list data type
#
################################################################################


################################################################################
# database handling
################################################################################


# open GAP database (variant 1: read)
#
# DESCRIPTION
# - argument 1: database name including version suffix
# - return val: database I/O handle
#
proc OpenDatabaseR {DbSpec} {

  # split name to database and version
  if {[set dot [string last . $DbSpec]] == -1} {
    puts stderr "ERROR: invalid database name '$DbSpec'"
    exit 1
  }
  set DbPathName [string range $DbSpec 0 [expr $dot-1]]
  set DbVersion  [string range $DbSpec [expr $dot+1] end]

  # DB file should exist
  if {! [file exists $DbSpec]} {
    puts stderr "ERROR: specified database '$DbSpec' doesn't exist"
    exit 1
  }

  # open DB, return handle
  capture {set io [open_db -name $DbPathName -version $DbVersion -create 0 -access r]} dummy
  if {$io == ""} {
    puts stderr "ERROR: unable to open the database '$DbPathName.$DbVersion'"
    exit 1
  }
  return $io
}


# open GAP database (variant 2: read/write, exist)
#
# DESCRIPTION
# - a former version of this procedure copied the DB to version X and
#   used this as a working copy. However, an existing DB version X could
#   be an active working copy and the original DB doesn't give a hint
#   by an existing 'DB.BUSY' file.
#
# - argument 1: database name including version suffix
# - return val: database I/O handle
#
# DEBUG, CHANGES, ADDITIONS
# - making a backup copy of the database before opening would be a good
#   idea.
#
proc OpenDatabaseRW {DbSpec} {
  global DbPathName DbVersion
  global read_only

  # split name to database and version
  if {[set dot [string last . $DbSpec]] == -1} {
    puts "ERROR: Invalid database name '$DbSpec'"
    exit 1
  }
  set DbPathName [string range $DbSpec 0 [expr $dot-1]]
  set DbVersion  [string range $DbSpec [expr $dot+1] end]

  # DB file should exist
  if {! [file exists "$DbSpec"]} {
    puts "ERROR: Specified database '$DbSpec' doesn't exist"
    exit 1
  }

  # DB busy?
  if {[file exists "$DbSpec.BUSY"]} {
    puts "ERROR: Database is busy. Cannot do changings."
    exit 1
  }

  # open DB, return handle
  set io [open_db -name $DbPathName -version $DbVersion -create 0 -access rw]
  if {"$io" == "" || $read_only == 1} {
    puts "ERROR: Couldn't open database '$DbPathName.$DbVersion' - exiting."
    exit 1
  }
  return $io
}


# open GAP database (variant 3: read/write, exist/create)
#
# DESCRIPTION
# - a former version of this procedure copied the DB to version X and
#   used this as a working copy. However, an existing DB version X could
#   be an active working copy and the original DB doesn't give a hint
#   by an existing 'DB.BUSY' file.
# - making a backup copy of the database before opening would be a good
#   idea.
#
# - argument 1: database name including version suffix
# - return val: database I/O handle
#
proc OpenDatabaseRWc {name} {
  global DbPathName DbVersion
  global read_only

  # split name to database and version
  if {[set CtPos [string last . $name]] == -1} {
    puts stderr "ERROR: invalid database name '$name'"
    exit 1
  }
  set DbPathName [string range $name 0 [expr $CtPos-1]]
  set DbVersion  [string range $name [expr $CtPos+1] end]

  # DB exists, open it
  if {[file exists $name]} {

    # DB busy?
    if {[file exists $name.BUSY]} {
      puts stderr "ERROR: database is busy. Cannot do changings"
      exit 1
    }

    # open DB
    set io [open_db -name $DbPathName -version $DbVersion -create 0 -access rw]

  # create new DB
  } else {
    set io [open_db -name $DbPathName -version $DbVersion -create 1 -access rw]
  }

  # no success?
  if {$io == "" || $read_only == 1} {
    puts stderr "ERROR: unable to open/create database '$DbPathName.$DbVersion'"
    exit 1
  }

  return $io
}


################################################################################
# report
################################################################################


# Returns a string describing the database
# Also (as this is a common place) bumps at maxseq as needed.
#
proc DatabaseInfo {io} {
  global maxseq
  global DbSpec

  set db [io_read_database $io]
  set nc [keylget db num_contigs]
  set nr [keylget db num_readings]
  set tcl [db_info t_contig_length $io]
  set trl [db_info t_read_length $io]
  set maxgel [keylget db max_gel_len]

  set RepStr ""
  append RepStr "============================================================\n"
  append RepStr [format "Database Name       %40s\n"  "$DbSpec"]
  append RepStr [format "Database size       %8d      Max reading length %7d\n" \
                [keylget db actual_db_size] $maxgel]
  append RepStr [format "No. Readings        %8d      No. Contigs        %7d\n" \
                $nr $nc]
  append RepStr [format "No. Singlets        %8d      No. Templates      %7d\n" \
                [llength [unattached_readings -io $io]] [keylget db Ntemplates]]
  append RepStr [format "No. Annotations     %8d                            \n" \
                [keylget db Nannotations]]
  append RepStr [format "No. Clones          %8d      No. Vectors        %7d\n" \
                [keylget db Nclones]      [keylget db Nvectors]]
  append RepStr [format "Total contig length %8d      Average length    %8.1f\n"\
                $tcl [expr $nc ? (double($tcl)/$nc) : 0]]
  append RepStr [format "Total characters in readings                    %12d\n" \
                $trl]
  append RepStr [format "Average reading characters per consensus character  %8.2f\n" \
                [expr $tcl ? (double($trl)/$tcl) : 0]]
  append RepStr [format "Average used length of reading                      %8.2f\n" \
                [expr $nr ? (double($trl)/$nr) : 0]]
  append RepStr "============================================================\n"

  return $RepStr
}


################################################################################
# lists
################################################################################


# create list of read names for a contig specifier
#
# DESCRIPTION
# - this function makes use of recursion for stepwise extension of the
#   requested list (1. list of contigs, 2. list of readings)
#
# - argument 1: contig specifier which may be
#               {allcontigs}  produces a list containing all readings
#               allreadings   produces a list containing all readings
#               allcontigs    produces a list containing all contigs
#               [contig_id]   contig identifier as either /#\d+/ or name of
#                             any contributing read
#
# proc ListGet {name} {
# code see $STADLIB/gap/list_proc.tcl


# convert plain text read list to list data type
#
# DESCRIPTION
# - argument 1: plain text read list
# - return val: list (data type) of reads
#
proc PlainToList {ListPlain} {
  set ListData ""

  # remove strings following the read identifier
  foreach line [split $ListPlain "\n"] {
    regsub { .*$} $line {} liner
    lappend ListData $liner
  }

  # return data
  return $ListData
}
