# >>>>>>>>>>>>>>>>>>>>>>>>> mbs module file <<<<<<<<<<<<<<<<

package mbs_routines;
require Exporter;

@ISA    = qw(Exporter);
@EXPORT = qw($MBS_ROUTINES_VERSION);
@EXPORT_OK = qw(ask_question check_answer get_taxid message_out
                formatEXP_Tag calc_pad_pos save_file get_gap_seq
                done_and_logged
               );

$MBS_ROUTINES_VERSION = "2.79";

##################################################################
# >>>>>>>>>>>>>>>>>>>>>>>>>  ask_question  <<<<<<<<<<<<<<<<<<<<<<<<
##################################################################

=pod

=head1 ask_question

B<ask_question> asks questions and offers a default answer.

=head2 IN

 In goes
 1. the question
 2. the default value
 3. "err" for printing to STDERR, other to STDOUT
 4. if wanted a line lenght for the question.

=head2 OUT

the answer (or the default value [2] if none given)

=cut


sub ask_question
{  # in goes the question an the default value and if wanted a question line length
   use strict;
   my ($question, $default_answer, $print_to, $line_len) = @_;
   
   my $space = '';
   if (defined $line_len)
   {
      my $space_num = $line_len - ((length ($question . $default_answer)) + 3);
      if ($space_num < 0)
      {
         $space = '';
      }
      else
      {
         $space = ' ' x $space_num;
      }
   }
   
   if ($print_to eq 'err')
   {
      printf STDERR "%s%s [%s]: ", $question, $space, $default_answer;
   }
   else
   {
      printf "%s%s [%s]: ", $question, $space, $default_answer;
   }
   
   my $u_answ = <STDIN>;
   chomp $u_answ;
   if ($u_answ eq '')
   {
      $u_answ = $default_answer;
   }
   
   return($u_answ);
}  # ask_question


##################################################################
# >>>>>>>>>>>>>>>>>>>>>>>  calc_pad_pos  <<<<<<<<<<<<<<<<<<<<<<<<
##################################################################

sub calc_pad_pos
{  # in goes padded seq, pos, and may be the good seq Type
   use strict;
   use Defaults;
   my ($padded_seq, $pos, $seq_types) = @_;
   my ($pads) = 0;
   
   $seq_types = $SeqTypes{all} unless defined $seq_types;
   
   my $OK_bases = (uc ($seq_types)) . (lc ($seq_types));
   
   while ($padded_seq =~ m/[^$OK_bases]/og and (pos $padded_seq) <= ($pos+$pads))
   {
      $pads++;
   }
   return ($pos + $pads);
}  # end calc_pad_pos


#########################################################################
# >>>>>>>>>>>>>>>>>>>>>>>>>>  check_answer  <<<<<<<<<<<<<<<<<<<<<<<<<<<<
#########################################################################

=pod

=head1 check_answer

B<check_answer> tests for validity of a value

Wenn Antwort = ? wird was gueltig waere ausgedruckt.

=head2 IN

 In goes
 1. the given value
 2. what is valid, could be Hash ref
 
=head2 OUT

the given value [1] if it is valid, else undef

 Version: 1.2   Date: 11.8.2005

=cut

sub check_answer
{  # in goes the given answer and what are valid answers
   my ($antwort, $gueltig) = @_;
   chomp $antwort;
   
   if (ref $gueltig eq 'HASH')
   {
      if ($antwort =~ /$gueltig->{$antwort}/ and $gueltig->{$antwort})
      {
         return ($antwort);
      }
      elsif ($antwort eq '?')
      {
         my $vlid = $gueltig->{$antwort};
         $vlid =~ s/^\^//;
         $vlid =~ s/\$$//;
         print "VALID: $vlid\n";
         return();
      }
      else
      {
         print "Invalid!\n";
         return ();
      }
   }
   else
   {
      if ($antwort eq '?')
      {
         my $vlid = $gueltig;
         $vlid =~ s/^\^//;
         $vlid =~ s/\$$//;
         print "VALID: $vlid\n";
         return();
      }
      elsif ($antwort =~ /$gueltig/ and $gueltig)
      {  # $gueltig = undef matches all!!
         return ($antwort);
      }
      else
      {
         print "Invalid!\n";
         return ();
      }
   }
}  # end check_answer

##################################################################
# >>>>>>>>>>>>>>>>>>>>>  done_and_logged  <<<<<<<<<<<<<<<<<<<<<<<
##################################################################

sub done_and_logged
{ # $path, @text
   use strict;
   my ($log_file, @log_text) = @_;
   open(LOG_FILE, ">>$log_file");
   print LOG_FILE @log_text;
   close(LOG_FILE);
}


##################################################################
# >>>>>>>>>>>>>>>>>>>>>>  formatEXP_Tag  <<<<<<<<<<<<<<<<<<<<<<
##################################################################

=pod

=head1 formatEXP_Tag

B<formatEXP_Tag> formatiert EXP Tag

=head2 IN

 In goes
 1. anonHash = {'record'   => TC or TG or ...
                'typ'      => $tag_type,
                'strand'   => $dir,
                'start'    => $st,
                'end'      => $end,
                'comment'  => ('db_xref=IMBGC2:12345', 'db_xref=taxon:4932',
                               'sonstige comments als Array-Elemente',
                              ),
                oder
                'comment'  => 'string',
               }
 
=head2 OUT
 
 1. formtierter String oder ''

=cut

sub formatEXP_Tag
{
   my ($tagHash) = @_;
   my $String;
   my $blank = ' ';
   
   # seperate records
   if ($tagHash->{record} =~ /^T(C|G)$/)
   {  # TC oder TG tag
      $String = $tagHash->{record} . ($blank x 3) . $tagHash->{typ} . ' ' .
                $tagHash->{strand} . ' ' .
                $tagHash->{start} . '..' . $tagHash->{end} . "\n";
      
      if (ref($tagHash->{comment}) eq 'ARRAY')
      {
         foreach my $entry (@{$tagHash->{comment}})
         {
            $String .= $tagHash->{record} . ($blank x 8) . $entry . "\n";
         }
      }
      elsif (ref($tagHash->{comment}) eq 'HASH')
      {
         foreach my $k (keys (%{$tagHash->{comment}}))
         {
            foreach my $v (@{$tagHash->{comment}->{$k}})
            {
               $String .= $tagHash->{record} . ($blank x 8) . ">$k=$v\n";
            }
         }
         #warn "Can't handle HASH as comment $String\n";
      }
      else
      {
         $String .= $tagHash->{record} . ($blank x 8) . $tagHash->{comment} . "\n";
      }
      
      return($String);
   }
   else
   {
      warn "Unknown record, can't do this\n";
      return();
   }
}  # end formatEXP_Tag

#########################################################################
# >>>>>>>>>>>>>>>>>>>>>>>>>>  get_gap_seq  <<<<<<<<<<<<<<<<<<<<<<<<<<<<
#########################################################################

sub get_gap_seq
{  # in goes gap_project name
   my ($gap_project) = @_;
   
   my $cmd = "gapsequence.tcl -g $gap_project -o $gap_project.cons -f X |";
   #my $cmd = "exportconsensus.tcl -c 2 $gap_project $gap_project.cons";
   open(ER, "$cmd");
   while (my $er = <ER>)
   {  # parse Messages
      print STDERR "$er";
   }
   close (ER);
   
   my ($rHseq, $rHseq_padded) = get_exp_seq("$gap_project.cons");
   
   return ($rHseq, $rHseq_padded);
}  # end get_gap_seq


#########################################################################
# >>>>>>>>>>>>>>>>>>>>>>>>>>  get_taxid  <<<<<<<<<<<<<<<<<<<<<<<<<<<<
#########################################################################

=pod

=head1 get_taxid

Gets the taxonomic ID of a given scientific name from the NCBI 
taxonomy database.

=head2 IN

 1. The scientific name

=head2 OUT

The taxonomic ID

=cut

sub get_taxid
{  # 
   use LWP::Simple;
   
   my ($scientific_name) = @_;
   
   my @name = split /\s+/, $scientific_name;
   my $host = 'http://www.ncbi.nih.gov/entrez/eutils/';
   my $path = 'esearch.fcgi?db=taxonomy&term=' . (join '+', @name);

   my $URL  = $host . $path;
   
   my $content = get($URL);
   
   #<Item Name="ScientificName" Type="String">Homo sapiens</Item>
   my @CONTENT = split(/\n/, $content);
   foreach my $line (@CONTENT)
   {
      if ($line =~ m|<Id>(.+)</Id>|)
      {
         return ($1);
      }
   }
   print STDERR "unknown species\n";
   return(0);
}  # end get_taxid


####################################################################
# >>>>>>>>>>>>>>>>>>>>  message_out  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<
####################################################################

=pod

=head1 message_out

B<message_out> Prints a message to STDERR :

Saving file test .......................
================================== 40 ==

=head2 IN

 In goes
 1. printf format string
 2. Line length
 3. List to be printed

 eg.: message_out('%s %0.2f', 60, ['NumFloat:', 1]);
 eg.: message_out('%s', 60, 'Hoppe hoppe Reiter');
 
=head2 OUT
 1. Prints message

v. 1.0, 24.11.2005

=cut

sub message_out
{
   my ($fmt, $lineLen, $msgs) = @_;
   
   my $out;
   if (ref $msgs eq 'ARRAY') {
      $out = sprintf $fmt, @$msgs;
   }
   else {
      $out = sprintf $fmt, $msgs;
   }
   
   my $n_dots = ($lineLen - (length $out));
   my $dots   = '.' x $n_dots;
   print STDERR $out, $dots, ' ';
   return(1);
}  # end message_out


##################################################################
# >>>>>>>>>>>>>>>>>>>>>>>>>  save_file  <<<<<<<<<<<<<<<<<<<<<<<<
##################################################################

=pod

=head1 save_file

B<save_file> 

=head2 IN

 In goes
 1. Filename
 2. deref Array or String of contents
 
=head2 OUT

 Out goes
 1. 1 for success, 0 for failure

=cut

sub save_file
{  # in goes $filename and what to save \@xx
   use strict;
   my ($outfile, $file_contens) = @_;
   
   unless (open(RECORD, ">$outfile")) { warn "Can't write to $outfile!\n"; return (0);}
   if (ref $file_contens eq 'ARRAY')
   {
      print RECORD @$file_contens;
      close (RECORD);
      return (1);
   }
   elsif (ref $file_contens eq '')
   {
      print RECORD $file_contens;
      close (RECORD);
      return (1);
   }
   else
   {
      print STDERR "ERROR saving\n";
      close (RECORD);
      return (0);
   }
}  # save_file

1;
